#! /usr/bin/tclsh
###############################################################################
# Test Case         : tc_conf_wrptp_mfg_006                                   #
# Test Case Version : 1.1                                                     #
# Component Name    : ATTEST WRPTP CONFORMANCE TEST SUITE                     #
# Module Name       : Message Format Group (MFG)                              #
#                                                                             #
# Title             : WRPTP Signaling message - WR Slave - transport is over  #
#                     IPv4/UDP.                                               #
#                                                                             #
# Purpose           : To verify that a WRPTP enabled device sends WRPTP       #
#                     Signaling messages (SLAVE_PRESENT, LOCKED, CALIBRATE    #
#                     and CALIBRATED) in correct format when its port is WR   #
#                     Slave and transport is over IPv4/UDP.                   #
#                                                                             #
# Reference         : White Rabbit Specification v2.0 July 2011, Clause 6.5.3 #
#                     Pages 28 and 29                                         #
#                                                                             #
# Conformance Type  : MUST                                                    #
#                                                                             #
###############################################################################
#                                                                             #
# Test Setup        : 001                                                     #
# Test Topology     : 1                                                       #
#                                                                             #
###############################################################################
#                                                                             #
# Ladder Diagram    :                                                         #
#                                                                             #
#          TEE                                                   DUT          #
#           |                                                     |           #
#           |                                        <Enable PTP> | P1        #
#           |                       <Enable PTP with BC/OC clock> |           #
#           |                    <Clock mode = One-step/Two-step> |           #
#           |                     <Transport Protocol = IPv4/UDP> |           #
#           |            <Configure Priority1 = X, Priority2 = Y> | P1        #
#           |                                      <Enable WRPTP> | P1        #
#           |                   <Configure wrConfig = WR_M_AND_S> | P1        #
#           |      <Configure default values for knownDeltaTx and | P1        #
#           |                                       knownDeltaRx> |           #
#           |                                                     |           #
#           | WRPTP ANNOUNCE [messageType = 0x0B,                 |           #
#           | domainNumber = DN1, grandmasterPriority1 = X - 1,   |           #
#           | tlvType = 0x0003, wrMessageId = 0x2000,             |           #
#           | wrConfig = 0x3, calibrated = 1, wrModeOn = 0]       |           #
#           | (sendCount = INFINITY)                              |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |            WRPTP SLAVE_PRESENT [messageType = 0x0C, |           #
#           |               domainNumber = DN1, tlvType = 0x0003, |           #
#           |                               wrMessageId = 0x1000] |           #
#           |                         {sourcePortIdentity = SPI1} |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | WRPTP LOCK [messageType = 0x0C,                     |           #
#           | domainNumber = DN1, targetPortIdentity = SPI1,      |           #
#           | tlvType = 0x0003, wrMessageId = 0x1001]             |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                   WRPTP LOCKED [messageType = 0x0C, |           #
#           |               domainNumber = DN1, tlvType = 0x0003, |           #
#           |                               wrMessageId = 0x1002] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           | WRPTP CALIBRATE [messageType = 0x0C,                |           #
#           | domainNumber = DN1, targetPortIdentity = SPI1,      |           #
#           | tlvType = 0x0003, wrMessageId = 0x1003,             |           #
#           | calSendPattern = FALSE, calRetry = 0,               |           #
#           | calPeriod = 3000us]                                 |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           | WRPTP CALIBRATED [messageType = 0x0C,               |           #
#           | domainNumber = DN1, targetPortIdentity = SPI1,      |           #
#           | tlvType = 0x0003, wrMessageId = 0x1004,             |           #
#           | deltaTx = 0, deltaRx = 0]                           |           #
#        T1 |-->>-------------------------------------------------| P1        #
#           |                                                     |           #
#           |                WRPTP CALIBRATE [messageType = 0x0C, |           #
#           |               domainNumber = DN1, tlvType = 0x0003, |           #
#           |                               wrMessageId = 0x1003] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#           |               WRPTP CALIBRATED [messageType = 0x0C, |           #
#           |               domainNumber = DN1, tlvType = 0x0003, |           #
#           |                               wrMessageId = 0x1004] |           #
#        T1 |-------------------------------------------------<<--| P1        #
#           |                                                     |           #
#                                                                             #
# Legends           :                                                         #
#                                                                             #
#     TEE       = Test Execution Engine                                       #
#     DUT       = Device Under Test                                           #
#     T1        = TEE's port 1                                                #
#     P1        = DUT's port 1                                                #
#     PTP       = Precision Time Protocol                                     #
#     WRPTP     = White Rabbit Precision Time Protocol                        #
#     OC        = Ordinary Clock                                              #
#     BC        = Boundary Clock                                              #
#     DN1       = Domain Number 1                                             #
#                                                                             #
###############################################################################
#                                                                             #
# Procedure         :                                                         #
#                                                                             #
# (Initial Part)                                                              #
#                                                                             #
# Step 1 : Initialization of DUT                                              #
#     i.   Enable DUT's port P1.                                              #
#    ii.   Enable PTP on port P1.                                             #
#   iii.   Enable PTP globally with device type as Boundary/Ordinary clock.   #
#    iv.   Configure clock mode as One-step/Two-step.                         #
#     v.   Configure Network Transport Protocol as IPv4/UDP.                  #
#    vi.   Configure default values for Priority1 = X and Priority2 = Y.      #
#   vii.   Enable WRPTP on port P1.                                           #
#  viii.   Configure wrConfig = WR_M_AND_S.                                   #
#    ix.   Configure default values for knownDeltaTx and knownDeltaRx on P1.  #
#                                                                             #
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
#                                                                             #
# (Part 1)                                                                    #
#                                                                             #
# Step 3 : Send periodic WRPTP ANNOUNCE messages on the port P1 with following#
#          parameters.                                                        #
#                                                                             #
#              PTP Header                                                     #
#                  messageType          = 0x0B                                #
#                  domainNumber         = DN1                                 #
#                  grandmasterPriority1 = X - 1                               #
#              TLV                                                            #
#                  tlvType              = 0x0003                              #
#                  wrMessageId          = 0x2000                              #
#                  wrConfig             = 0x3                                 #
#                  calibrated           = 1                                   #
#                  wrModeOn             = 0                                   #
#                                                                             #
# Step 4 : Wait until completion of BMCA and verify that DUT transmits WRPTP  #
#          SLAVE_PRESENT message on the port P1 with following parameters and #
#          store sourcePortIdentity as SPI1.                                  #
#                                                                             #
#               Ethernet Header                                               #
#                 1) Source MAC                = Unicast MAC                  #
#                 2) Destination MAC           = Unicast MAC or               #
#                                                01:1B:19:00:00:00            #
#                 3) EtherType                 = IPv4 (0x800)                 #
#                                                                             #
#               IPv4 Fields                                                   #
#                 4) IP Protocol = 17 (UDP)                                   #
#                 5) Destination IP = 224.0.1.129                             #
#                 6) Source IP = Unicast IP                                   #
#                 7) Checksum = Valid                                         #
#                                                                             #
#               UDP Fields                                                    #
#                 8) UDP Destination Port = 320 (General Message)             #
#                 9) Checksum = Valid                                         #
#                                                                             #
#               PTP Header                                                    #
#                10) transportSpecific         = 0 or 1, Reserved: 2-F        #
#                                                (4 bits)                     #
#                11) messageType               = 0x0C (4 bits)                #
#                12) Reserved Bits (1)         = 0 (4 bits)                   #
#                13) Version                   = 2 (4 bits)                   #
#                14) messageLength             = 56 (2 octets)                #
#                15) domainNumber              = 0 - 255, Reserved: 128 - 255 #
#                16) Reserved Bits (2)         = 0 (8 bits)                   #
#                17) flagField                 = 0x0000 - 0xFFFF              #
#                18) correctionField           = 0  (8 octets)                #
#                19) Reserved Bits (3)         = 0 (32 bits)                  #
#                20) sourcePortIdentity                                       #
#                     a)clockIdentity          = 8 octets                     #
#                     b)portNumber             = 2 octets                     #
#                21) sequenceId                = 0 - 65535                    #
#                22) controlField              = 5                            #
#                23) logMessageInterval        = 127                          #
#                24) targetPortIdentity        = non-zero (10 octets)         #
#               TLV Header                                                    #
#                25) tlvType                   = 0x0003                       #
#                26) lengthField               = 8 (2 octets)                 #
#                27) OrganizationId            = 0x080030                     #
#                28) magicNumber               = 0xDEAD                       #
#                29) versionNumber             = 0x01                         #
#                30) wrMessageId               = 0x1000                       #
#                                                                             #
# Step 5 : Send WRPTP LOCK message on port T1 with following parameters.      #
#                                                                             #
#              PTP Header                                                     #
#                  messageType          = 0x0C                                #
#                  domainNumber         = DN1                                 #
#                  targetPortIdentity   = SPI1                                #
#              TLV                                                            #
#                  tlvType              = 0x0003                              #
#                  wrMessageId          = 0x1001                              #
#                                                                             #
# Step 6 : Verify that DUT transmits WRPTP LOCKED message on the port P1 with #
#          following parameters.                                              #
#                                                                             #
#               Ethernet Header                                               #
#                 1) Source MAC                = Unicast MAC                  #
#                 2) Destination MAC           = Unicast MAC or               #
#                                                01:1B:19:00:00:00            #
#                 3) EtherType                 = IPv4 (0x800)                 #
#                                                                             #
#               IPv4 Fields                                                   #
#                 4) IP Protocol = 17 (UDP)                                   #
#                 5) Destination IP = 224.0.1.129                             #
#                 6) Source IP = Unicast IP                                   #
#                 7) Checksum = Valid                                         #
#                                                                             #
#               UDP Fields                                                    #
#                 8) UDP Destination Port = 320 (General Message)             #
#                 9) Checksum = Valid                                         #
#                                                                             #
#               PTP Header                                                    #
#                10) transportSpecific         = 0 or 1, Reserved: 2-F        #
#                                                (4 bits)                     #
#                11) messageType               = 0x0C (4 bits)                #
#                12) Reserved Bits (1)         = 0 (4 bits)                   #
#                13) Version                   = 2 (4 bits)                   #
#                14) messageLength             = 56 (2 octets)                #
#                15) domainNumber              = 0 - 255, Reserved: 128 - 255 #
#                16) Reserved Bits (2)         = 0 (8 bits)                   #
#                17) flagField                 = 0x0000 - 0xFFFF              #
#                18) correctionField           = 0 (8 octets)                 #
#                19) Reserved Bits (3)         = 0 (32 bits)                  #
#                20) sourcePortIdentity                                       #
#                     a)clockIdentity          = 8 octets                     #
#                     b)portNumber             = 2 octets                     #
#                21) sequenceId                = 0 - 65535                    #
#                22) controlField              = 5                            #
#                23) logMessageInterval        = 127                          #
#                24) targetPortIdentity        = non-zero (10 octets)         #
#               TLV Header                                                    #
#                25) tlvType                   = 0x0003                       #
#                26) lengthField               = 8 (2 octets)                 #
#                27) OrganizationId            = 0x080030                     #
#                28) magicNumber               = 0xDEAD                       #
#                29) versionNumber             = 0x01                         #
#                30) wrMessageId               = 0x1002                       #
#                                                                             #
# Step 7 : Send WRPTP CALIBRATE message on port T1 with following parameters. #
#                                                                             #
#              PTP Header                                                     #
#                  messageType          = 0x0C                                #
#                  domainNumber         = DN1                                 #
#                  targetPortIdentity   = SPI1                                #
#              TLV                                                            #
#                  tlvType              = 0x0003                              #
#                  wrMessageId          = 0x1003                              #
#                  calSendPattern       = FALSE                               #
#                  calRetry             = 0                                   #
#                  calPeriod            = 3000us                              #
#                                                                             #
# Step 8 : Send WRPTP CALIBRATED message on port T1 with following parameters #
#                                                                             #
#              PTP Header                                                     #
#                  messageType          = 0x0C                                #
#                  domainNumber         = DN1                                 #
#                  targetPortIdentity   = SPI1                                #
#              TLV                                                            #
#                  tlvType              = 0x0003                              #
#                  wrMessageId          = 0x1004                              #
#                  deltaTx              = 0                                   #
#                  deltaRx              = 0                                   #
#                                                                             #
# Step 9 : Verify that DUT transmits WRPTP CALIBRATE message on the port P1   #
#          with following parameters.                                         #
#                                                                             #
#               Ethernet Header                                               #
#                 1) Source MAC                = Unicast MAC                  #
#                 2) Destination MAC           = Unicast MAC or               #
#                                                01:1B:19:00:00:00            #
#                 3) EtherType                 = IPv4 (0x800)                 #
#                                                                             #
#               IPv4 Fields                                                   #
#                 4) IP Protocol = 17 (UDP)                                   #
#                 5) Destination IP = 224.0.1.129                             #
#                 6) Source IP = Unicast IP                                   #
#                 7) Checksum = Valid                                         #
#                                                                             #
#               UDP Fields                                                    #
#                 8) UDP Destination Port = 320 (General Message)             #
#                 9) Checksum = Valid                                         #
#                                                                             #
#               PTP Header                                                    #
#                10) transportSpecific         = 0 or 1, Reserved: 2-F        #
#                                                (4 bits)                     #
#                11) messageType               = 0x0C (4 bits)                #
#                12) Reserved Bits (1)         = 0 (4 bits)                   #
#                13) Version                   = 2 (4 bits)                   #
#                14) messageLength             = 62 (2 octets)                #
#                15) domainNumber              = 0 - 255, Reserved: 128 - 255 #
#                16) Reserved Bits (2)         = 0 (8 bits)                   #
#                17) flagField                 = 0x0000 - 0xFFFF              #
#                18) correctionField           = 0 (8 octets)                 #
#                19) Reserved Bits (3)         = 0 (32 bits)                  #
#                20) sourcePortIdentity                                       #
#                     a)clockIdentity          = 8 octets                     #
#                     b)portNumber             = 2 octets                     #
#                21) sequenceId                = 0 - 65535                    #
#                22) controlField              = 5                            #
#                23) logMessageInterval        = 127                          #
#                24) targetPortIdentity        = non-zero (10 octets)         #
#               TLV Header                                                    #
#                25) tlvType                   = 0x0003                       #
#                26) lengthField               = 14 (2 octets)                #
#                27) OrganizationId            = 0x080030                     #
#                28) magicNumber               = 0xDEAD                       #
#                29) versionNumber             = 0x01                         #
#                30) wrMessageId               = 0x1003                       #
#                31) calSendPattern            = 0x0 (Warning, if 0x1)        #
#                32) calRetry                  = 1 octet                      #
#                33) calPeriod                 = 4 octets                     #
#                                                                             #
# Step 10 : Verify that DUT transmits WRPTP CALIBRATED message on the port P1 #
#           with following parameters.                                        #
#                                                                             #
#               Ethernet Header                                               #
#                 1) Source MAC                = Unicast MAC                  #
#                 2) Destination MAC           = Unicast MAC or               #
#                                                01:1B:19:00:00:00            #
#                 3) EtherType                 = IPv4 (0x800)                 #
#                                                                             #
#               IPv4 Fields                                                   #
#                 4) IP Protocol = 17 (UDP)                                   #
#                 5) Destination IP = 224.0.1.129                             #
#                 6) Source IP = Unicast IP                                   #
#                 7) Checksum = Valid                                         #
#                                                                             #
#               UDP Fields                                                    #
#                 8) UDP Destination Port = 320 (General Message)             #
#                 9) Checksum = Valid                                         #
#                                                                             #
#               PTP Header                                                    #
#                10) transportSpecific         = 0 or 1, Reserved: 2-F        #
#                                                (4 bits)                     #
#                11) messageType               = 0x0C (4 bits)                #
#                12) Reserved Bits (1)         = 0 (4 bits)                   #
#                13) Version                   = 2 (4 bits)                   #
#                14) messageLength             = 72 (2 octets)                #
#                15) domainNumber              = 0 - 255, Reserved: 128 - 255 #
#                16) Reserved Bits (2)         = 0 (8 bits)                   #
#                17) flagField                 = 0x0000 - 0xFFFF              #
#                18) correctionField           = 0 (8 octets)                 #
#                19) Reserved Bits (3)         = 0 (32 bits)                  #
#                20) sourcePortIdentity                                       #
#                     a)clockIdentity          = 8 octets                     #
#                     b)portNumber             = 2 octets                     #
#                21) sequenceId                = 0 - 65535                    #
#                22) controlField              = 5                            #
#                23) logMessageInterval        = 127                          #
#                24) targetPortIdentity        = non-zero (10 octets)         #
#               TLV Header                                                    #
#                25) tlvType                   = 0x0003                       #
#                26) lengthField               = 24 (2 octets)                #
#                27) OrganizationId            = 0x080030                     #
#                28) magicNumber               = 0xDEAD                       #
#                29) versionNumber             = 0x01                         #
#                30) wrMessageId               = 0x1004                       #
#                31) deltaTx                   = 8 octets                     #
#                32) deltaRx                   = 8 octets                     #
#                                                                             #
###############################################################################
#                                                                             #
# History        Date        Author        Addition/Alteration                #
#                                                                             #
# 1.0          Aug/2018      CERN          Initial                            #
# 1.1          Jan/2019      CERN          Modified waiting timeout to receive#
#                                          SLAVE_PRESENT, LOCKED, CALIBRATE   #
#                                          and CALIBRATED messages.           #
#                                                                             #
###############################################################################
# Copyright (c) 2018 - 2019 CERN                                              #
###############################################################################

global dut_log_msg
global tee_log_msg
global session_id

####################### LOCAL VARIABLES ARE DECLARED HERE #####################

set tc_def    "To verify that a WRPTP enabled device sends WRPTP Signaling\
               messages (SLAVE_PRESENT, LOCKED, CALIBRATE and CALIBRATED)\
               in correct format when its port is WR Slave and transport\
               is over IPv4/UDP."

########################### START - INITIALIZATION ############################

set tee_port_1                     $::tee_port_num_1
set dut_port_1                     $::dut_port_num_1
set vlan_id                        $::wrptp_vlan_id
set dut_ip                         $::dut_test_port_ip
set dut_dest_ip                    $::dut_test_port_ip
set tee_dest_ip                    $::tee_test_port_ip
set ptp_comm_model                 $::ptp_comm_model
set ptp_ethtype                    $::PTP_ETHTYPE
set arp_timeout                    $::ARP_TIMEOUT

set domain                         $::DEFAULT_DOMAIN
set wrptp_tlv                      $::ORG_EXT_TLV

set ann_msg_id                     $::wrMessageID(ANN_SUFIX)
set slave_present_msg_id           $::wrMessageID(SLAVE_PRESENT)
set lock_msg_id                    $::wrMessageID(LOCK)
set locked_msg_id                  $::wrMessageID(LOCKED)
set calibrate_msg_id               $::wrMessageID(CALIBRATE)
set calibrated_msg_id              $::wrMessageID(CALIBRATED)

set infinity                       $::INFINITY
set wrconfig                       $::WRCONFIG(WR_MASTER_AND_SLAVE)
set calibrated_state               $::WRPTP_PORT_STATE(CALIBRATED)

set announce_timeout               [expr $::wrptp_announce_timeout + 60]
set signal_timeout                 $::wrptp_signal_timeout

set bmca_time                      $::wrptp_announce_timeout

set calibrated                     1
set step                           1

########################### END - INITIALIZATION ##############################

if { $::ptp_trans_type == $::PTP_TRANS_TYPE(IEEE) } {

    TC_ABORT -reason "This test case is applicable only for transport type IPv4/UDP."\
                       -tc_def  $tc_def

}

wrptp::display_interface_details

wrptp::dut_Initialization

proc cleanup {} {

    wrptp::dut_cleanup_setup_001

}

##############################################################################
# NOTE : Cleanup procedure must be called once both the sessions have been   #
#        opened                                                              #
##############################################################################

TC_CLEAN

### (Initial Part) ###

###############################################################################
# Step 1 : Initialization of DUT                                              #
#     i.   Enable DUT's port P1.                                              #
#    ii.   Enable PTP on port P1.                                             #
#   iii.   Enable PTP globally with device type as Boundary/Ordinary clock.   #
#    iv.   Configure clock mode as One-step/Two-step.                         #
#     v.   Configure Network Transport Protocol as IPv4/UDP.                  #
#    vi.   Configure default values for Priority1 = X and Priority2 = Y.      #
#   vii.   Enable WRPTP on port P1.                                           #
#  viii.   Configure wrConfig = WR_M_AND_S.                                   #
#    ix.   Configure default values for knownDeltaTx and knownDeltaRx on P1.  #
###############################################################################

STEP $step

LOG -level 0 -msg "$dut_log_msg(INIT_SETUP1)"

if {![wrptp::dut_configure_setup_001]} {

    LOG -msg "$dut_log_msg(INIT_SETUP1_F)"

    TC_CLEAN_AND_ABORT -reason "$dut_log_msg(INIT_SETUP1_F)"\
                      -tc_def   $tc_def
    return 0
}

incr step

###############################################################################
# Step 2 : Initialization of TEE                                              #
#     i.   Add port T1 at TEE.                                                #
###############################################################################

STEP $step

LOG -level 0 -msg "$tee_log_msg(INIT)"

wrptp::Initialization

set tee_mac       $::TEE_MAC
set tee_ip        $::tee_test_port_ip

if {$ptp_comm_model == "Unicast"} {

    if {![WRPTP_UNICAST_PRE_REQUISITE_CONFIGURATION\
                        -port_num       $tee_port_1\
                        -tee_mac        $tee_mac\
                        -tee_ip         $tee_ip\
                        -dut_ip         $dut_ip\
                        -vlan_id        $vlan_id\
                        -dut_mac        tee_dest_mac\
                        -timeout        $arp_timeout]} {

        TEE_CLEANUP

        TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ARP_F)"\
                           -tc_def  $tc_def

        return 0
    }

} else {
    set tee_dest_ip             $::PTP_E2E_IP
    set tee_dest_mac            $::PTP_E2E_MAC
}

if {![pltLib::create_filter_wrptp_pkt\
              -port_num                $tee_port_1\
              -ether_type              $ptp_ethtype\
              -vlan_id                 $vlan_id\
              -ids                     id1]} {

    TC_CLEAN_AND_ABORT -reason "$tee_log_msg(FILTER_F)"\
                       -tc_def  $tc_def

    return 0
}

set filter_id [lindex $id1 end]

wrptp::reset_capture_stats\
        -port_num           $tee_port_1

incr step

###############################################################################
# Step 3 : Send periodic WRPTP ANNOUNCE messages on the port T1 with following#
#          parameters.                                                        #
#                                                                             #
#              PTP Header                                                     #
#                  messageType          = 0x0B                                #
#                  domainNumber         = DN1                                 #
#                  grandmasterPriority1 = X - 1                               #
#              TLV                                                            #
#                  tlvType              = 0x0003                              #
#                  wrMessageId          = 0x2000                              #
#                  wrConfig             = 0x3                                 #
#                  calibrated           = 1                                   #
#                  wrModeOn             = 0                                   #
###############################################################################

STEP $step

LOG -level 0 -msg "$tee_log_msg(ANNOUNCE_TX_P1)"

set gm_priority1 [expr $::wrptp_priority1 - 1]

if {![wrptp::send_announce\
            -port_num         $tee_port_1\
            -dest_mac         $tee_dest_mac\
            -src_mac          $tee_mac\
            -count            $infinity\
            -src_ip           $tee_ip\
            -dest_ip          $tee_dest_ip\
            -domain_number    $domain\
            -gm_priority1     $gm_priority1\
            -tlv_type         $wrptp_tlv\
            -message_id       $ann_msg_id\
            -wrconfig         $wrconfig\
            -calibrated       $calibrated]} {

     TEE_CLEANUP

     TC_CLEAN_AND_ABORT -reason "$tee_log_msg(ANNOUNCE_TX_P1_F)"\
                        -tc_def  $tc_def
     return 0

}

incr step

###############################################################################
# Step 4 : Wait until completion of BMCA and verify that DUT transmits WRPTP  #
#          SLAVE_PRESENT message on the port P1 with following parameters and #
#          store sourcePortIdentity as SPI1.                                  #
#                                                                             #
#               Ethernet Header                                               #
#                 1) Source MAC                = Unicast MAC                  #
#                 2) Destination MAC           = Unicast MAC or               #
#                                                01:1B:19:00:00:00            #
#                 3) EtherType                 = IPv4 (0x800)                 #
#                                                                             #
#               IPv4 Fields                                                   #
#                 4) IP Protocol = 17 (UDP)                                   #
#                 5) Destination IP = 224.0.1.129                             #
#                 6) Source IP = Unicast IP                                   #
#                 7) Checksum = Valid                                         #
#                                                                             #
#               UDP Fields                                                    #
#                 8) UDP Destination Port = 320 (General Message)             #
#                 9) Checksum = Valid                                         #
#                                                                             #
#               PTP Header                                                    #
#                10) transportSpecific         = 0 or 1, Reserved: 2-F        #
#                                                (4 bits)                     #
#                11) messageType               = 0x0C (4 bits)                #
#                12) Reserved Bits (1)         = 0 (4 bits)                   #
#                13) Version                   = 2 (4 bits)                   #
#                14) messageLength             = 56 (2 octets)                #
#                15) domainNumber              = 0 - 255, Reserved: 128 - 255 #
#                16) Reserved Bits (2)         = 0 (8 bits)                   #
#                17) flagField                 = 0x0000 - 0xFFFF              #
#                18) correctionField           = 0  (8 octets)                #
#                19) Reserved Bits (3)         = 0 (32 bits)                  #
#                20) sourcePortIdentity                                       #
#                     a)clockIdentity          = 8 octets                     #
#                     b)portNumber             = 2 octets                     #
#                21) sequenceId                = 0 - 65535                    #
#                22) controlField              = 5                            #
#                23) logMessageInterval        = 127                          #
#                24) targetPortIdentity        = non-zero (10 octets)         #
#               TLV Header                                                    #
#                25) tlvType                   = 0x0003                       #
#                26) lengthField               = 8 (2 octets)                 #
#                27) OrganizationId            = 0x080030                     #
#                28) magicNumber               = 0xDEAD                       #
#                29) versionNumber             = 0x01                         #
#                30) wrMessageId               = 0x1000                       #
#                                                                             #
###############################################################################

STEP $step

LOG -level 0 -msg "$tee_log_msg(BMCA_SLAVE_PRESENT_RX_P1_V)"

set bmca_signal_timeout  [expr $::wrptp_present_timeout_recv + $bmca_time]

if {![wrptp::check_signal\
           -port_num                  $tee_port_1\
           -filter_id                 $filter_id\
           -domain_number             $domain\
           -tlv_type                  $wrptp_tlv\
           -timeout                   $bmca_signal_timeout\
           -recvd_src_port_number     recvd_src_port_number\
           -recvd_src_clock_identity  recvd_src_clock_identity\
           -message_id                $slave_present_msg_id]} {

    TEE_CLEANUP

    TC_CLEAN_AND_FAIL   -reason "DUT does not send WRPTP SLAVE_PRESENT message\
                                 in correct format when transport is over IPv4/UDP."\
                        -tc_def  $tc_def

    return 0

}

incr step

###############################################################################
# Step 5 : Send WRPTP LOCK message on port T1 with following parameters.      #
#                                                                             #
#              PTP Header                                                     #
#                  messageType          = 0x0C                                #
#                  domainNumber         = DN1                                 #
#                  targetPortIdentity   = SPI1                                #
#              TLV                                                            #
#                  tlvType              = 0x0003                              #
#                  wrMessageId          = 0x1001                              #
###############################################################################

STEP $step

LOG -level 0 -msg "$tee_log_msg(LOCK_TX_P1)"

if {![wrptp::send_signal\
           -port_num               $tee_port_1\
           -src_mac                $tee_mac\
           -dest_mac               $tee_dest_mac\
           -src_ip                 $tee_ip\
           -dest_ip                $tee_dest_ip\
           -domain_number          $domain\
           -target_port_number     $recvd_src_port_number\
           -target_clock_identity  $recvd_src_clock_identity\
           -tlv_type               $wrptp_tlv\
           -message_id             $lock_msg_id]} {

     TEE_CLEANUP

     TC_CLEAN_AND_ABORT -reason "$tee_log_msg(LOCK_TX_P1_F)"\
                        -tc_def  $tc_def

     return 0

}

incr step

###############################################################################
# Step 6 : Verify that DUT transmits WRPTP LOCKED message on the port P1 with #
#          following parameters.                                              #
#                                                                             #
#               Ethernet Header                                               #
#                 1) Source MAC                = Unicast MAC                  #
#                 2) Destination MAC           = Unicast MAC or               #
#                                                01:1B:19:00:00:00            #
#                 3) EtherType                 = IPv4 (0x800)                 #
#                                                                             #
#               IPv4 Fields                                                   #
#                 4) IP Protocol = 17 (UDP)                                   #
#                 5) Destination IP = 224.0.1.129                             #
#                 6) Source IP = Unicast IP                                   #
#                 7) Checksum = Valid                                         #
#                                                                             #
#               UDP Fields                                                    #
#                 8) UDP Destination Port = 320 (General Message)             #
#                 9) Checksum = Valid                                         #
#                                                                             #
#               PTP Header                                                    #
#                10) transportSpecific         = 0 or 1, Reserved: 2-F        #
#                                                (4 bits)                     #
#                11) messageType               = 0x0C (4 bits)                #
#                12) Reserved Bits (1)         = 0 (4 bits)                   #
#                13) Version                   = 2 (4 bits)                   #
#                14) messageLength             = 56 (2 octets)                #
#                15) domainNumber              = 0 - 255, Reserved: 128 - 255 #
#                16) Reserved Bits (2)         = 0 (8 bits)                   #
#                17) flagField                 = 0x0000 - 0xFFFF              #
#                18) correctionField           = 0 (8 octets)                 #
#                19) Reserved Bits (3)         = 0 (32 bits)                  #
#                20) sourcePortIdentity                                       #
#                     a)clockIdentity          = 8 octets                     #
#                     b)portNumber             = 2 octets                     #
#                21) sequenceId                = 0 - 65535                    #
#                22) controlField              = 5                            #
#                23) logMessageInterval        = 127                          #
#                24) targetPortIdentity        = non-zero (10 octets)         #
#               TLV Header                                                    #
#                25) tlvType                   = 0x0003                       #
#                26) lengthField               = 8 (2 octets)                 #
#                27) OrganizationId            = 0x080030                     #
#                28) magicNumber               = 0xDEAD                       #
#                29) versionNumber             = 0x01                         #
#                30) wrMessageId               = 0x1002                       #
#                                                                             #
###############################################################################

STEP $step

LOG -level 0 -msg "$tee_log_msg(LOCKED_RX_P1_V)"

if {![wrptp::check_signal\
           -port_num                $tee_port_1\
           -domain_number           $domain\
           -tlv_type                $wrptp_tlv\
           -message_id              $locked_msg_id\
           -timeout                 $::wrptp_locked_timeout_recv\
           -filter_id               $filter_id]} {

    TEE_CLEANUP

    TC_CLEAN_AND_FAIL   -reason "DUT does not send WRPTP LOCKED message in\
                                 correct format when transport is over IPv4/UDP."\
                        -tc_def  $tc_def

    return 0
}

wrptp::reset_capture_stats\
        -port_num           $tee_port_1

incr step

###############################################################################
# Step 7 : Send WRPTP CALIBRATE message on port T1 with following parameters. #
#                                                                             #
#              PTP Header                                                     #
#                  messageType          = 0x0C                                #
#                  domainNumber         = DN1                                 #
#                  targetPortIdentity   = SPI1                                #
#              TLV                                                            #
#                  tlvType              = 0x0003                              #
#                  wrMessageId          = 0x1003                              #
#                  calSendPattern       = FALSE                               #
#                  calRetry             = 0                                   #
#                  calPeriod            = 3000us                              #
###############################################################################

STEP $step

LOG -level 0 -msg "$tee_log_msg(CALIBRATE_TX_P1)"

set cal_retry    0
set cal_period   3000

if {![wrptp::send_signal\
             -port_num               $tee_port_1\
             -src_ip                 $tee_ip\
             -dest_ip                $tee_dest_ip\
             -src_mac                $tee_mac\
             -dest_mac               $tee_dest_mac\
             -domain_number          $domain\
             -target_port_number     $recvd_src_port_number\
             -target_clock_identity  $recvd_src_clock_identity\
             -tlv_type               $wrptp_tlv\
             -cal_retry              $cal_retry\
             -cal_period             $cal_period\
             -message_id             $calibrate_msg_id]} {

    TEE_CLEANUP

    TC_CLEAN_AND_ABORT -reason "$tee_log_msg(CALIBRATE_TX_P1_F)"\
                       -tc_def  $tc_def

    return 0

}

incr step

###############################################################################
# Step 8 : Send WRPTP CALIBRATED message on port T1 with following parameters.#
#                                                                             #
#              PTP Header                                                     #
#                  messageType          = 0x0C                                #
#                  domainNumber         = DN1                                 #
#                  targetPortIdentity   = SPI1                                #
#              TLV                                                            #
#                  tlvType              = 0x0003                              #
#                  wrMessageId          = 0x1004                              #
#                  deltaTx              = 0                                   #
#                  deltaRx              = 0                                   #
###############################################################################

STEP $step

LOG -level 0 -msg "$tee_log_msg(CALIBRATED_TX_P1)"

set delta_tx    0
set delta_rx    0

if {![wrptp::send_signal\
             -port_num               $tee_port_1\
             -src_ip                 $tee_ip\
             -dest_ip                $tee_dest_ip\
             -src_mac                $tee_mac\
             -dest_mac               $tee_dest_mac\
             -domain_number          $domain\
             -target_port_number     $recvd_src_port_number\
             -target_clock_identity  $recvd_src_clock_identity\
             -tlv_type               $wrptp_tlv\
             -delta_tx               $delta_tx\
             -delta_rx               $delta_rx\
             -message_id             $calibrated_msg_id]} {

    TEE_CLEANUP

    TC_CLEAN_AND_ABORT -reason "$tee_log_msg(CALIBRATED_TX_P1_F)"\
                       -tc_def  $tc_def

    return 0

}

incr step

###############################################################################
# Step 9 : Verify that DUT transmits WRPTP CALIBRATE message on the port P1   #
#          with following parameters.                                         #
#                                                                             #
#               Ethernet Header                                               #
#                 1) Source MAC                = Unicast MAC                  #
#                 2) Destination MAC           = Unicast MAC or               #
#                                                01:1B:19:00:00:00            #
#                 3) EtherType                 = IPv4 (0x800)                 #
#                                                                             #
#               IPv4 Fields                                                   #
#                 4) IP Protocol = 17 (UDP)                                   #
#                 5) Destination IP = 224.0.1.129                             #
#                 6) Source IP = Unicast IP                                   #
#                 7) Checksum = Valid                                         #
#                                                                             #
#               UDP Fields                                                    #
#                 8) UDP Destination Port = 320 (General Message)             #
#                 9) Checksum = Valid                                         #
#                                                                             #
#               PTP Header                                                    #
#                10) transportSpecific         = 0 or 1, Reserved: 2-F        #
#                                                (4 bits)                     #
#                11) messageType               = 0x0C (4 bits)                #
#                12) Reserved Bits (1)         = 0 (4 bits)                   #
#                13) Version                   = 2 (4 bits)                   #
#                14) messageLength             = 62 (2 octets)                #
#                15) domainNumber              = 0 - 255, Reserved: 128 - 255 #
#                16) Reserved Bits (2)         = 0 (8 bits)                   #
#                17) flagField                 = 0x0000 - 0xFFFF              #
#                18) correctionField           = 0 (8 octets)                 #
#                19) Reserved Bits (3)         = 0 (32 bits)                  #
#                20) sourcePortIdentity                                       #
#                     a)clockIdentity          = 8 octets                     #
#                     b)portNumber             = 2 octets                     #
#                21) sequenceId                = 0 - 65535                    #
#                22) controlField              = 5                            #
#                23) logMessageInterval        = 127                          #
#                24) targetPortIdentity        = non-zero (10 octets)         #
#               TLV Header                                                    #
#                25) tlvType                   = 0x0003                       #
#                26) lengthField               = 14 (2 octets)                #
#                27) OrganizationId            = 0x080030                     #
#                28) magicNumber               = 0xDEAD                       #
#                29) versionNumber             = 0x01                         #
#                30) wrMessageId               = 0x1003                       #
#                31) calSendPattern            = 0x0 (Warning, if 0x1)        #
#                32) calRetry                  = 1 octet                      #
#                33) calPeriod                 = 4 octets                     #
#                                                                             #
###############################################################################

STEP $step

LOG -level 0 -msg "$tee_log_msg(CALIBRATE_RX_P1_V)"

if {![wrptp::check_signal\
           -port_num                $tee_port_1\
           -domain_number           $domain\
           -tlv_type                $wrptp_tlv\
           -message_id              $calibrate_msg_id\
           -timeout                 $::wrptp_calibration_timeout_recv\
           -filter_id               $filter_id]} {

    TEE_CLEANUP

    TC_CLEAN_AND_FAIL   -reason "DUT does not send WRPTP CALIBRATE message in\
                                 correct format when transport is over IPv4/UDP."\
                        -tc_def  $tc_def

    return 0
}

incr step

###############################################################################
# Step 10 : Verify that DUT transmits WRPTP CALIBRATED message on the port P1 #
#           with following parameters.                                        #
#                                                                             #
#               Ethernet Header                                               #
#                 1) Source MAC                = Unicast MAC                  #
#                 2) Destination MAC           = Unicast MAC or               #
#                                                01:1B:19:00:00:00            #
#                 3) EtherType                 = IPv4 (0x800)                 #
#                                                                             #
#               IPv4 Fields                                                   #
#                 4) IP Protocol = 17 (UDP)                                   #
#                 5) Destination IP = 224.0.1.129                             #
#                 6) Source IP = Unicast IP                                   #
#                 7) Checksum = Valid                                         #
#                                                                             #
#               UDP Fields                                                    #
#                 8) UDP Destination Port = 320 (General Message)             #
#                 9) Checksum = Valid                                         #
#                                                                             #
#               PTP Header                                                    #
#                10) transportSpecific         = 0 or 1, Reserved: 2-F        #
#                                                (4 bits)                     #
#                11) messageType               = 0x0C (4 bits)                #
#                12) Reserved Bits (1)         = 0 (4 bits)                   #
#                13) Version                   = 2 (4 bits)                   #
#                14) messageLength             = 72 (2 octets)                #
#                15) domainNumber              = 0 - 255, Reserved: 128 - 255 #
#                16) Reserved Bits (2)         = 0 (8 bits)                   #
#                17) flagField                 = 0x0000 - 0xFFFF              #
#                18) correctionField           = 0 (8 octets)                 #
#                19) Reserved Bits (3)         = 0 (32 bits)                  #
#                20) sourcePortIdentity                                       #
#                     a)clockIdentity          = 8 octets                     #
#                     b)portNumber             = 2 octets                     #
#                21) sequenceId                = 0 - 65535                    #
#                22) controlField              = 5                            #
#                23) logMessageInterval        = 127                          #
#                24) targetPortIdentity        = non-zero (10 octets)         #
#               TLV Header                                                    #
#                25) tlvType                   = 0x0003                       #
#                26) lengthField               = 24 (2 octets)                #
#                27) OrganizationId            = 0x080030                     #
#                28) magicNumber               = 0xDEAD                       #
#                29) versionNumber             = 0x01                         #
#                30) wrMessageId               = 0x1004                       #
#                31) deltaTx                   = 8 octets                     #
#                32) deltaRx                   = 8 octets                     #
#                                                                             #
###############################################################################

STEP $step

LOG -level 0 -msg "$tee_log_msg(CALIBRATED_RX_P1_V)"

if {![wrptp::check_signal\
           -port_num                $tee_port_1\
           -domain_number           $domain\
           -tlv_type                $wrptp_tlv\
           -message_id              $calibrated_msg_id\
           -timeout                 $::wrptp_calibrated_timeout_recv\
           -filter_id               $filter_id]} {

    TEE_CLEANUP

    TC_CLEAN_AND_FAIL   -reason "DUT does not send WRPTP CALIBRATED message in\
                                 correct format when transport is over IPv4/UDP."\
                        -tc_def  $tc_def

     return 0

}

TEE_CLEANUP

TC_CLEAN_AND_PASS  -reason "DUT sends WRPTP Signaling messages (SLAVE_PRESENT,\
                            LOCKED, CALIBRATE and CALIBRATED) in correct format\
                            when its port is WR Slave and transport is over\
                            IPv4/UDP."\
                   -tc_def  $tc_def

return 1

############################### END OF TESTCASE ################################
